#pragma once

/**
 * GenerationInfo.h - Bitcoin poscript generator
 *
 * Copyright (C)2015 The bitgen developers <bitgen@openmailbox.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */


#include <string>
#include <vector>
#include <list>
#include "bigint.h"
#include "extraInfo.h"

#include "GenerationParameters.h"

class GenerationInfoBase
{
public:
	GenerationInfoBase(const std::string& inTime, const std::string& nodeName, const std::string& inApplicationName, const std::string& inVersionNumber) : 
		time(inTime),
		name(nodeName),
		//newNum(inNewNum),
		applicationName(inApplicationName), 
		versionNumber(inVersionNumber)
			{}

	void setName(const std::string& name);

	std::string getNodeName() const
	{
		return name;
	}

	std::string getTime() const
	{
		return time;
	}



	void setVersionNumber(const std::string& inversionNumber)
	{
		versionNumber = inversionNumber;
	}

	std::string getVersionNumber() const
	{
		return versionNumber;
	}

	std::string getApplicationName() const
	{
		return applicationName;
	}


private:
	std::string name;
	std::string time;
	
	
	//int newNum = 0;

	std::string versionNumber;
	std::string applicationName;

};




class GenerationInfoCore : public GenerationInfoBase
{
public:
	GenerationInfoCore(const GenerationInfoBase& base, const int inNewNum, const GenerationParameters& inParameters) : 
		GenerationInfoBase(base), params(inParameters), accountName(chooseAccountNameOrNumber(inNewNum, inParameters.getAccountNameInternal(), inParameters.useConter())),
				newNum(inNewNum)
			{
			}


	static std::string chooseAccountNameOrNumber(const int newNum, const std::string& name, const bool useConter);


	GenerationParameters params;


	int getNewNum() const
	{
		return newNum;
	}


	std::string getAccountName() const
	{
		return accountName;
	}

private:

	std::string accountName;

//	std::string name;
//	std::string time;
	
	
	int newNum = 0;
	
};


class GenerationInfo : public GenerationInfoCore
{
public:	
	GenerationInfo(const GenerationInfoCore& paramCore, const std::list<ExtraInfo>& inGenerationExtra,
		const std::string& inGenerator) : 
		GenerationInfoCore(paramCore), generationExtra(inGenerationExtra),
		generator(inGenerator)
		{
		}

/*
	void setDerivationPath(const std::string& inDerivationPath)
	{
		derivationPath = inDerivationPath;
	}


	std::string getDerivationPath() const
	{
		return derivationPath;
	}
*/

	void setInhibitFirstPrivkeyLine(const bool inInhibitFirstPrivkeyLine)
	{
		inhibitFirstPrivkeyLine = inInhibitFirstPrivkeyLine;
	}

	bool getInhibitFirstPrivkeyLine() const
	{
		return inhibitFirstPrivkeyLine;
	}




	void setStaticXor(const BigInt<256>& inStaticXor)
	{
		staticXor = inStaticXor;
	}

	BigInt<256> getStaticXor() const
	{
		return staticXor;
	}



	void setIsPad(const bool inIsPad)
	{
		isPad = inIsPad;
	}

	bool getIsPad() const
	{
		return isPad;
	}


	
	void setMnemonic(const std::string& inMnemonic)
	{
		mnemonic = inMnemonic;
	}

	std::string getMnemonic() const
	{
		return mnemonic;
	}


	std::string getGenerator() const
	{
		return generator;
	}

	void setMethod(const std::string& inMethod)
	{
		method = inMethod;
	}

	std::string getMethod() const
	{
		return method;
	}



	//void setCashAddress(const std::string& inCashAddress)
	//{
	//	cashAddress = inCashAddress;
	//}

	std::string getCashAddress() const
	{
		return cashAddress;
	}
	
	//void setCashAddressRead(const std::string& inCashAddressRead)
	//{
	//	cashAddressRead = inCashAddressRead;
	//}

	std::string getCashAddressRead() const
	{
		return cashAddressRead;
	}




	const std::list<ExtraInfo>& getGenerationExtra() const
	{
		return generationExtra;
	}


private:

	static void verifyNumWords(const int numWords);


	//std::string versionNumber;
	//std::string applicationName;

public:
	std::list<ExtraInfo> generationExtra;
private:


	std::string generator;
	std::string method;


	std::string cashAddressRead;
	std::string cashAddress;

	bool isPad = false;
	bool inhibitFirstPrivkeyLine = false;

	//secret
	std::string mnemonic;
	BigInt<256> staticXor;

	std::string derivationPath;
};


class AddressGenerationInfo : public GenerationInfo
{
public:
	AddressGenerationInfo(const std::string& inPubaddr, const std::string& inPrivaddr, const std::string& inPrivaddrName, const GenerationInfo& inParams, const std::string inDerivationPath = "") : 
		GenerationInfo(inParams), pubaddr(inPubaddr), privaddr(inPrivaddr), privaddrName(inPrivaddrName), derivationPath(inDerivationPath) {}

	std::string getDerivationPath() const
	{
		return derivationPath;
	}


	const std::string pubaddr;		//Todo: Private
	const std::string privaddr;		//Todo: Private
	const std::string privaddrName;

	const std::string derivationPath;

};



std::string getNodeName();





