#ifndef GENERATION_METHODS_H
#define GENERATION_METHODS_H

/**
 * GenerationMethods.h - Bitcoin poscript generator
 *
 * Copyright (C)2015 The bitgen developers <bitgen@openmailbox.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 
#include <vector>
#include <string>
#include <list>

#include "GenerationParameters.h"
#include "bitcoinKeyPair.h"
#include "extraInfo.h"

#include "mnemonic.h"

#include "ExtraOptionsRandom.h"

#include "GenerationInfo.h"

#include "xmrPrivKey.h"

class GenerationMethods
{
public:	
	static void generateZcashz(const std::vector<std::string>& args, const std::string& thetime);

	//static void deprecated_generateMonero(const std::vector<std::string>& args, const std::string& thetime);	//Todo: Remove
	static void generateMoneroPrivSpend(const std::vector<std::string>& args, const std::string& thetime);
	static void generateMoneroRandom(const std::vector<std::string>& args, const std::string& thetime);
	static void generateMoneroHash(const std::vector<std::string>& args, const std::string& thetime);
	static void generateMoneroDice(const std::vector<std::string>& args, const std::string& thetime);
	static void generateMoneroHex(const std::vector<std::string>& args, const std::string& thetime);
	static void generateMoneroBrainWallet(const std::vector<std::string>& args, const std::string& thetime);
	static void generateMoneroOtpParts(const std::vector<std::string>& args, const std::string& thetime);
	static void generateMoneroOtpTotal(const std::vector<std::string>& args, const std::string& thetime);

	static void deriveMoneroAddresses(const std::vector<std::string>& args, const std::string& thetime);
	static void deriveMoneroAddressesFromPrivSpend(const std::vector<std::string>& args, const std::string& thetime);
	static void deriveMoneroAddressesFromPrivView(const std::vector<std::string>& args, const std::string& thetime);




	static void generateMoneroFromMnemonic(const std::vector<std::string>& args, const std::string& thetime);
	static void infoAddress(const std::vector<std::string>& args, const std::string& thetime);


	static void generateBrainWallet(const std::vector<std::string>& args, const std::string& thetime);
	static void generateHash(const std::vector<std::string>& args, const std::string& thetime);
	static void generateHex(const std::vector<std::string>& args, const std::string& thetime);
	static void generateBulk(const std::vector<std::string>& args, const std::string& thetime);
	static void generateWithDice(const std::vector<std::string>& args, const std::string& thetime);
	static void generateFromMnemonic(const std::vector<std::string>& args, const std::string& thetime);
	static void generateRandom(const std::vector<std::string>& args, const std::string& thetime);
	static void generateOtpParts(const std::vector<std::string>& args, const std::string& thetime);
	static void generateOtpTotal(const std::vector<std::string>& args, const std::string& thetime);
	
	
	static void generateFromWif(const std::vector<std::string>& args, const std::string& thetime);
	
	static void printRandomSourceWarning(const std::string generator = "/dev/random");
	
	static std::string generateOneFileContents(const BitcoinKeyPair& keypair, const bool addPubkey, std::string& customString);
	
	
	static void saveAllFiles(const AddressGenerationInfo& allParams);
	
	//const std::string& pubaddr, const std::string& cashAddress, const std::string& cashAddressRead,
	//	const std::string& privaddr, const std::string& genMethod,
	//	const std::list<ExtraInfo>& generationExtra, const std::string& thetime, const GenerationParameters& params,
	//	const std::string& mnemonic, 
	//	const int newNum = 0, const bool isPad = false, const BigInt<256> staticXor = BigInt<256>(), const bool inhibitFirstPrivkeyLine = false
	//);
	
	static void saveTheFilesInFolders(const AddressGenerationInfo& allInfo, const std::string& privPath, const std::string& pubPath);
	//const std::string& pubaddr, const std::string& cashAddress, const std::string& cashAddressRead,
	//	const std::string& privaddr, const std::string& genMethod,
	//	const std::list<ExtraInfo>& generationExtra, const std::string& thetime, const GenerationParameters& params,
	//	const std::string& mnemonic, const int newNum, const bool isPad, const BigInt<256> staticXor,
	//	const std::string& privPath, const std::string& pubPath, const bool inhibitFirstPrivkeyLine = false)	;
	
	static void saveTheOTPPublicFilesInFolders(const AddressGenerationInfo& AllInfo, const std::string& pubPath);
		//const std::string& pubaddr, 
		//const std::string& thetime, const GenerationParameters& params,
		//const int newNum, 
		//const std::string& pubPath);
	

	static void generateBinaryRandomNumber(const std::vector<std::string>& args, const std::string& thetime);
	

	static int getNextNumberIfConfigured(const GenerationParameters& params);

	
private:

	static void handleOtpPrivKey(const GenerationParameters& params, const BigInt<256>& staticXor, const int numPads, const BigInt<256>& privKey, const std::string& thetime);

	static void generateXmrFromPrivSpendKey(const BigInt<256>& privKeySpend, GenerationParameters& params, const std::string& thetime, 
		const std::string& genMethod, std::list<ExtraInfo>& privateGenerationExtra);

	static void generateXmrFromThePrivSpendkey(const XmrPrivSpendKey& privSpendKey, GenerationParameters& params, const std::string& thetime, 
		const std::string& genMethod, std::list<ExtraInfo>& privateGenerationExtra);

	static void generateXmrFromRawPrivkey(const BigInt<256>& privKeyRaw, GenerationParameters& params, const std::string& thetime, 
		const std::string& genMethod, std::list<ExtraInfo>& generationExtra);


	static GenerationParameters parseCommandlineParams(const std::vector<std::string>& args, ExtraOptionsRandom& extraOptions);
	
	
	static bool parseXmrCommandlineParams(const std::vector<std::string>& args, const int minNumArgs, const int argStartPos, GenerationParameters& params, ExtraOptionsRandom& extraOptions);


	static void savePublicPadPart(const AddressGenerationInfo& allInfo, const std::string& pubPat);
		//const std::string& pubAddr, const std::string& thetime,
		//const GenerationParameters& params, const int newNum, const std::string& pubPath);

	static void savePrivatePadPart(const GenerationInfo& info, const std::list<BigInt<256> >& pads, 
		const std::list<std::string>& hashParts, std::vector<std::string>& privatePaths, const int totalPads);
		//const std::list<BigInt<256> >& pads, const std::string method,
		//const std::list<std::string>& hashParts, std::vector<std::string>& privatePaths, const std::string& pubAddr,
		//const std::string generator, const std::string& thetime, const int totalPads, const GenerationParameters& params,
		//const BigInt<256>& staticXor, const int newNum);


	static void splitPassphrase(const std::string& passphrase, std::list<std::string>& generationExtra);
};



BigInt<256> generateRandomWithMethod(const std::string& method, const std::string& extra, const std::string& extra2);

std::list<BigInt<256> > generatePadRandom(const int totalPads, const std::string& method, const std::string& extraVal);

std::list<std::string> splitHashStr(const int totNum, const std::string& totalHashVal);

std::list<BigInt<256> > getPadsFromCommandline(const std::vector<std::string>& args, bool& useStaticXorIfPresent);


#endif
