
#include "GenerationMethodsElectrum.h"
#include "GenerationMethodsExtended.h"

#include "randomUtils.h"
#include "random.h"
#include "keyDerivationFunction.h"

#include "bitgenVersion.h"


void GenerationMethodsElectrum::saveElectrumMnemonic(const GenerationInfo& info, const std::list<ExtraInfo>& extra, 
	const ExtendedPrivateKey& extendedPrivate)
{	
	//std::cout << "In saveElectrumMnemonic----------" << std::endl;
	
	//if(!mnemonic.electrum())
	//{
	//	throw std::string("Internal error, mnemonic not electrum type");
	//}
	
	
	std::cout << "Found seed: " << info.getMnemonic() << std::endl;	
	std::cout << std::endl;

	//const ExtendedPrivateKey extendedPrivate = info.getMnemonic().getPrivateFromElectrumMnemonic();	
	
	std::cout << "Extended private: " << extendedPrivate.toString() << std::endl;
	
	const ExtendedPublicKey extendedPub = extendedPrivate.getPub();	

	std::cout << "Extended pub: " << extendedPub.toString() << std::endl;
	std::cout << std::endl;
	

	std::list<ExtraInfo> generationExtra;
		
	const ExtraInfo extraInfo("Mnemonic", info.getMnemonic(), true);
	generationExtra.push_back(extraInfo);

	generationExtra.insert(generationExtra.end(), extra.begin(), extra.end());
	//generationExtra.insert(allInfo.getGenerationExtra().end(), allInfo.getGenerationExtra().begin(), allInfo.getGenerationExtra().end());

	//const GenerationInfoCore infoCore(thetime, getNodeName(), params);
	//GenerationInfo info(infoCore, generationExtra, 0, applicationName, versionNumber, "hex");
	
	//info.setDerivationPath(extendedPub.extendedData.chainString());
	
	const AddressGenerationInfo allInfo(extendedPub.toString(), extendedPrivate.toString(), "ExtPriv", info, extendedPub.extendedData.chainString());

	GenerationMethodsExtended::saveExtendedFiles(allInfo, extendedPub, true);
}



void GenerationMethodsElectrum::generateElectrumMnemonicInfo(const GenerationInfoCore infoCore,
	const BigInt<256>& randomNum, 
	const int numWords, 
	const std::string& genMethod, 
	const std::list<ExtraInfo>& extra
	)
{	
	const ElectrumMnemonic mnemonic = ElectrumMnemonic::generateElectrum(randomNum, numWords);

	std::list<ExtraInfo> generationExtra;
		
	//const ExtraInfo extraInfo("Mnemonic", mnemonic.toStr(), true);
	//generationExtra.push_back(extraInfo);

	generationExtra.insert(generationExtra.end(), extra.begin(), extra.end());

	GenerationInfo info(infoCore, generationExtra, genMethod);
	
	info.setMnemonic(mnemonic.toStr());
		
	const ExtendedPrivateKey extendedPrivate = mnemonic.getPrivateFromElectrumMnemonic();	
		
	saveElectrumMnemonic(info, extra, extendedPrivate);
}



void GenerationMethodsElectrum::createElectrumDice(const GenerationInfoCore infoCore, const int numWords, const std::string& diceStr)
{
	std::cout << "Creating electrum seed with " << numWords << " words from dice" << std::endl;
	
	const double numWordsDouble(numWords);
	
	const int numDiceNeeded = ((numWordsDouble * 11.0) / 2.63);
	
	if(diceStr.size() != numDiceNeeded)
	{
		std::cout << "Number dice needed: " << numDiceNeeded << ", got: " << diceStr.size() << std::endl;
		throw std::string("Not correct number dice digits");
	}
	
	const BigInt<256> randomNum = getRandomNumFrom6SideDice(diceStr, numDiceNeeded);		
	//std::cout << "Random: " << hexString(randomNum.getBinaryLong())	<< std::endl;
	//std::cout << "Non-zero digits: " << randomNum.numberBitsNeeded() << std::endl;
	
	
	std::list<ExtraInfo> extra;
	const ExtraInfo extraInfo("Dice", diceStr);	
	extra.push_back(extraInfo);
	
	//const GenerationInfoCore infoCore(thetime, getNodeName(), params);	
	
	generateElectrumMnemonicInfo(infoCore, randomNum, numWords, "dice", extra);
}




	//const std::string& thetime,
	//const GenerationParameters& params

void GenerationMethodsElectrum::createElectrumHex(const GenerationInfoCore infoCore, const int numWords, const std::string& hex)
{
	std::cout << "Creating electrum seed from hex" << std::endl;
	
	const int numWordsBound = (numWords * 11) * 2 / 8;
	const int numHexCharsNeeded = min(numWordsBound, 64);
	
	if(hex.size() < numHexCharsNeeded)
	{
		std::cout << "Num hex digits needed: " << numHexCharsNeeded << ", got: " << hex.size() << std::endl;
		throw std::string("Not enough hex digits");
	}
	else if(hex.size() > numHexCharsNeeded)
	{
		std::cout << "Num hex digits needed: " << numHexCharsNeeded << ", got: " << hex.size() << std::endl;
		throw std::string("Too many hex digits");
	}
	
	const std::string hexUpper = toUpper(hex);
		
	const BigInt<256> randomNum = BigInt<256>::fromHex(hexUpper);
	
	std::list<ExtraInfo> extra;
	const ExtraInfo extraInfo("Hex", hexUpper);	
	extra.push_back(extraInfo);

	generateElectrumMnemonicInfo(infoCore, randomNum, numWords, "hex", extra);
}




	//const std::string& thetime,
	//const GenerationParameters& params

void GenerationMethodsElectrum::createElectrumBrain(const GenerationInfoCore infoCore, const int numWords, const std::string& salt, const std::string& brainVal)
{
	const int minBrainStrSize(100);
	
	if(brainVal.size() < minBrainStrSize)
	{
		std::cout << "Brain string size: " << brainVal.size() << ", need: " << minBrainStrSize << std::endl;
		throw std::string("Brain string too short");
	}
	
	std::cout << "Creating electrum seed from brain string" << std::endl;
	
	std::cout << "Calculating privkey, this may take a while..." << std::endl;	
	const std::string kdfStr = bitgenArgon2Kdf(brainVal, salt);
	if(kdfStr.size() != 32)
	{
		throw std::string("KDF error");
	}
	const std::string sha = sha256(kdfStr);
	const BigInt<256> randomNum(sha, 'c');
		
			
	std::list<ExtraInfo> extra;
	const ExtraInfo extraInfo("Brain", brainVal, true);	
	extra.push_back(extraInfo);
	
	generateElectrumMnemonicInfo(infoCore, randomNum, numWords, "brain", extra);	
}




	//const std::string& thetime,
	//const GenerationParameters& params

void GenerationMethodsElectrum::createElectrumHash(const GenerationInfoCore infoCore, const int numWords, const std::string& hash)
{
	std::cout << "Creating electrum seed from hash" << std::endl;
	
	const BigInt<256> randomNum = hashStringToBigInt(hash);

	const std::list<ExtraInfo> extra;
	
	//const GenerationInfoCore infoCore(thetime, getNodeName(), params);	
	
	generateElectrumMnemonicInfo(infoCore, randomNum, numWords, "hash", extra);
}




void GenerationMethodsElectrum::createElectrumRandom(const GenerationInfoCore infoCore,	const int numWords)
{
	std::cout << "Creating electrum seed from /dev/random" << std::endl;
	
	BigInt<256> randomNum = getRandomNumber();		//Todo: Only get as much entropy that is needed with numWords
	//std::cout << "Random num: " << hexString(randomNum.getBinaryLong()) << std::endl;
	
	const std::list<ExtraInfo> extra;
		
	generateElectrumMnemonicInfo(infoCore, randomNum, numWords, "random", extra);
}





void GenerationMethodsElectrum::createElectrumMnemonic(const GenerationInfoCore infoCore, const std::string& mnemonicRaw, const std::list<ExtraInfo>& extra)
{
	std::cout << "Creating extended key from mnemonic" << std::endl;
	
	//Make sure we use lower case letters with single spaces.	
	//const std::string mnemonicStr = normalizeMnemonicStr(mnemonicRaw);
	const ElectrumMnemonic mnemonic(mnemonicRaw);
	const int numWords = mnemonic.numberWords();
	
	if(numWords > 24)
	{
		std::cout << "Number words incorrent, got: " << numWords << " words" << std::endl;
		throw std::string("Incorrect num words");
	}
	
	std::cout << "Normalized mnemonic: \"" << mnemonic.toStr() << "\"" << std::endl;

	//const GenerationInfoCore infoCore(thetime, getNodeName(), params);	
	std::list<ExtraInfo> generationExtra;

	//std::list<ExtraInfo> generationExtra;
		
	const ExtraInfo extraInfo("Mnemonic", mnemonic.toStr(), true);
	generationExtra.push_back(extraInfo);

	generationExtra.insert(generationExtra.end(), extra.begin(), extra.end());
	
		
	GenerationInfo info(infoCore, generationExtra, "hex");

	info.setMnemonic(mnemonic.toStr());


	const ExtendedPrivateKey extendedPrivate = mnemonic.getPrivateFromElectrumMnemonic();	

	saveElectrumMnemonic(info, extra, extendedPrivate);	
}









