#ifndef PRINT_POSTSCIPT_H
#define PRINT_POSTSCIPT_H

/**
 * printPS.h - Bitcoin poscript generator
 *
 * Copyright (C)2015 The bitgen developers <bitgen@openmailbox.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */


#include <set>
#include <vector>
#include <string>
#include <list>

#include "qrenc.h"

#include "language.h"
#include "extraInfo.h"
#include "bigint.h"

#include "pubAddress.h"
#include "ExtendedKey.h"
#include "GenerationParameters.h"
#include "GenerationInfo.h"

class Coordinate
{
public:
	Coordinate(const int inX, const int inY) : x(inX), y(inY) {}	
	
	const int x;
	const int y;	
};


//Todo: Move to file "postscriptGeneration.h"

class PsInfoFile
{
public:
	PsInfoFile(const std::string& inPublic, const std::string& inPrivate) :
		publicPs(inPublic), privatePs(inPrivate)  {}
	
	
	const std::string publicPs;
	const std::string privatePs;
};



class PostscriptLineGenerator
{
public:

static int publicLinenoNumToYpos(const int lineNum)
{
	const int yPosition(780 - lineNum * 20);
	return yPosition;
}




static int not_used_privateLinenoNumToYpos(const int lineNum)
{
	const int yPosition(200 - lineNum * 20);
	return yPosition;
}





//Todo: Member in PostscriptGenerator
static std::string lineBetweenPoints(const std::list<Coordinate>& coordinates)
{
	std::string ret;
	
	bool first(true);
	
	for(std::list<Coordinate>::const_iterator it = coordinates.begin() ;
		it != coordinates.end() ;
		++it)
	{
		const Coordinate& coord = (*it);
		
		ret += intToString(coord.x);
		ret += " ";
		ret += intToString(coord.y);
		if(first)
		{
			first = false;			
			ret += " moveto ";
		}
		else
		{
			ret += " lineto ";
		}			
		
	}
	
	ret += "closepath stroke\n";
	
	return ret;
}





//Todo: Member in PostscriptGenerator
static std::string lineBetween(const int x1, const int y1, const int x2, const int y2)
{
	std::string ret;
	
	ret += intToString(x1);	
	ret += " ";	
	ret += intToString(y1);	
	ret += " moveto ";	
	ret += intToString(x2);	
	ret += " ";	
	ret += intToString(y2);	
	
	ret += " lineto\n";

	return ret;
}


};



class PublicPostscriptInfo
{
public:

	PublicPostscriptInfo(const AddressGenerationInfo& allInfo, const bool inDisplayHashOfTime = false) :
		pubkey(allInfo.pubaddr), thetime(allInfo.getTime()), accountName(allInfo.getAccountName()), computerName(allInfo.getNodeName()), displayHashOfTime(inDisplayHashOfTime),
		newNum(allInfo.getNewNum())
		//Todo: inComputerName
		{}

	
	const std::string pubkey;
	const std::string pubkeyCashAddr;
	const std::string pubkeyCashAddrRead;
	
	const std::string thetime;
	const std::string accountName;	
	const std::string computerName;		
	const bool displayHashOfTime;

	const int newNum;
};



class PublicExtendedPostscriptInfo
{
public:
	PublicExtendedPostscriptInfo(const std::string& inNodeName,
		const std::string & inPubkey, const std::string& inThetime, const std::string& inAccountName,
		const bool inDisplayHashOfTime = false) : 
		nodeName(inNodeName), pubkey(inPubkey), thetime(inThetime), accountName(inAccountName), displayHashOfTime(inDisplayHashOfTime) {}
	
	const std::string nodeName;
	const std::string pubkey;
	const std::string thetime;
	const std::string accountName;	
	const bool displayHashOfTime;
};





class PrivatePostscriptInfo
{
public:
/*
	PrivatePostscriptInfo(const std::string& inPrivkey, const std::string& inMnemonic, 
		const std::string& inGenerationMethod, 
		const bool inCompressed, 
		const std::list<ExtraInfo>& inGenerationExtra, 
		const bool inIsPad = false, const int inPadNum = 0, const int inTotalPads = 0,
		const BigInt<256>& inStaticXor = BigInt<256>(), const bool inInhibitFirstPrivkeyLine = false) : 
			privkey(inPrivkey), mnemonic(inMnemonic), generationMethod(inGenerationMethod), 
			compressed(inCompressed), generationExtra(inGenerationExtra), isPad(inIsPad),
			padNum(inPadNum), totalPads(inTotalPads), staticXor(inStaticXor), inhibitFirstPrivkeyLine(inInhibitFirstPrivkeyLine) {}
*/


	PrivatePostscriptInfo(const AddressGenerationInfo& allInfo, const bool inIsPad = false, 
		const int inPadNum = 0, const int inTotalPads = 0) :
			privkey(allInfo.privaddr), privkeyName(allInfo.privaddrName), mnemonic(allInfo.getMnemonic()), generationMethod(allInfo.getGenerator()),
			compressed(allInfo.params.getCompressed()), generationExtra(allInfo.getGenerationExtra()), isPad(inIsPad),
			padNum(inPadNum), totalPads(inTotalPads), staticXor(allInfo.getStaticXor()), inhibitFirstPrivkeyLine(allInfo.getInhibitFirstPrivkeyLine()) {}
	


	bool hasBip39Mnemonic() const
	{
		//Todo: Remove hardcoded string
		if(generationMethod == "Random with HD first address derivation")
		{
			return true;
		}
				
		return false;
	}

	const std::string privkey;
	const std::string privkeyName;
	const std::string mnemonic;
	const std::string generationMethod;
	const bool compressed;
	const std::list<ExtraInfo> generationExtra;
	const bool isPad;
	const int padNum;
	const int totalPads;
	const BigInt<256> staticXor;
	const bool inhibitFirstPrivkeyLine;
};



class HierarcialInfo
{
public:
	HierarcialInfo(const std::string& inSignature, const std::string& inPubX, const std::string& inPubY) :
		signature(inSignature), pubX(inPubX), pubY(inPubY) {}
	
	const std::string signature;	
	const std::string pubX;
	const std::string pubY;
};







#include "postscriptGeneration.h"





#endif
