#ifndef XMR_MNEMONIC_H
#define XMR_MNEMONIC_H

/**
 * mnemonic.h - Bitcoin poscript generator
 *
 * Copyright (C)2015 The bitgen developers <bitgen@openmailbox.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <string>
#include <list>
#include <array>
#include <map>

#include "bigint.h"
#include "xmrPrivKey.h"


class XmrMnemonic
{
public:		
	explicit XmrMnemonic(const std::string& inMnemonic);
	explicit XmrMnemonic(const XmrPrivKey& inPrivkey);



	BigInt<256> toPrivKeyInt() const;	//Todo: Needed?

	XmrPrivSpendKey getSpendKey() const;

	
	std::string toStr() const { return mnemonic; }


private:
	void xmrUnittests();	//Todo: Remove

	
	static std::string bigIntToMnemonic(BigInt<264> total);
	static BigInt<264> mnemonicToBigInt(const std::string& mnemonic, std::string& checksumWord);
	
	static int wordToInt(std::map<std::string, int>& wordMap, const std::string& word);

	static std::list<std::string> splitToWords(std::string str);

	static std::string concatenateWords(const std::list<std::string>& words);


	static std::array<unsigned int, 256> crcLookupTable();

	static unsigned int crc(const std::string& str);

	static XmrMnemonic generateMnemonicFromKey(const BigInt<256>& privkey);

	static int getNumberTextWords(const std::string& mnemonicStr);


	static std::string trim(const std::string& mnemonic);
	static std::string normalizeWords(const std::string& str);
	
	static std::string checkWordsInWordlistAndNormalize(const std::string& mnemonic);

	static std::string getWordWithIndex(const std::string& mnemonic, const unsigned int computedIndex);

	static int computeChecksumIndex(const std::string& mnemonicStr);
	static void verifyChecksum(const std::string& mnemonicStr);

	static std::string getWordAtPosition25(const std::string& mnemonic);

	static std::string verifyAndNormalizeFullMnemonic(const std::string& mnemonicRaw);

	static constexpr long word_list_length = 1626;

	const std::string mnemonic;
};




#endif
