#ifndef XMR_PUBKEY_H
#define XMR_PUBKEY_H

/**
 * pubKey.h - Bitcoin poscript generator
 *
 * Copyright (C)2015 The bitgen developers <bitgen@openmailbox.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "bigintutil.h"
#include "xmrEC.h"
#include "pubAddress.h"


class XmrPubKey
{
public:

	explicit XmrPubKey(const XmrECPoint& inP) : 
		point(inP)
		{}


	std::string asBinary() const
	{
		BigInt<256> yVal(point.getY());
		
		const bool leastSignificantBit = point.getX()[0];
		if(leastSignificantBit)
		{
			yVal.setBit(255);
		}

		return yVal.getBinaryLong();						

	}


	BigInt<256> xyVal() const
	{
		BigInt<256> yVal(point.getY());		
		
		const bool leastSignificantBit = point.getX()[0];
		if(leastSignificantBit)
		{
			yVal.setBit(255);
		}
		
		return yVal;
	}


	std::string toString() const
	{
		return hexString(asBinary());
	}

	const XmrECPoint point;
	
private:	

};



#endif
